package cn.iocoder.yudao.module.member.mq.consumer.score.core;

import cn.iocoder.yudao.module.member.api.score.MemberUserScoreApi;
import cn.iocoder.yudao.module.member.api.score.dto.MemberUserScoreOperateReqDTO;
import cn.iocoder.yudao.module.member.dal.dataobject.scoreRule.ScoreRuleDO;
import cn.iocoder.yudao.module.member.dal.dataobject.user.MemberUserDO;
import cn.iocoder.yudao.module.member.enums.ScoreRuleTypeEnum;
import cn.iocoder.yudao.module.member.enums.ScoreSourceTypeEnum;
import cn.iocoder.yudao.module.member.mq.message.ScoreMessage;
import cn.iocoder.yudao.module.member.service.scoreLog.MemberUserScoreLogService;
import cn.iocoder.yudao.module.member.service.scoreRule.ScoreRuleService;
import cn.iocoder.yudao.module.member.service.user.MemberUserService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.HashMap;

/**
 * @author zhangfeng
 */
@Service
@Slf4j
public class RecommendStrategy extends AbstractScoreRuleStrategy {
    @Resource
    private MemberUserService memberUserService;

    public RecommendStrategy(ScoreRuleService scoreRuleService, MemberUserScoreLogService memberUserScoreLogService, MemberUserService memberUserService, MemberUserScoreApi memberUserScoreApi) {
        super(scoreRuleService, memberUserScoreLogService, memberUserService, memberUserScoreApi);
    }

    @Override
    public void addScore(ScoreMessage message) {
        log.info("Received Recommend message,referralCode:{}", message.getReferralCode());
        // 根据推荐码获取推荐用户id
        MemberUserDO memberUserDO = memberUserService.getUserByCode(message.getReferralCode());
        if (memberUserDO == null) {
            log.info("Recommend listener: can not find user by referralCode:{}", message.getReferralCode());
            return;
        }
        Long userId = memberUserDO.getId();
        ScoreRuleDO scoreRuleDO = scoreRuleService.getEnableScoreRuleByType(ScoreRuleTypeEnum.RECOMMEND);
        if (scoreRuleDO == null) {
            log.info("Recommend rule is not exist,userID:{}", userId);
            return;
        }
        // 校验累计最高分
        Integer userScoreTotalCount = getUserScoreTotalCount(scoreRuleDO.getId(), userId);
        if (scoreRuleDO.getMaxScoreTotal() > 0 && userScoreTotalCount >= scoreRuleDO.getMaxScoreTotal()) {
            log.info("Recommend listener: The user has reached the maximum score,userId:{},scoreRuleId:{}", userId, scoreRuleDO.getId());
            return;
        }
        try {
            HashMap<String, Object> map = new HashMap<>();
            map.put("userId", userId);
            map.put("userNameZh", memberUserDO.getNickname());
            map.put("userNameEn", memberUserDO.getEnglishName());
            String desensitizedPhoneNumber = memberUserDO.getMobile().replaceAll("(\\d{3})\\d{4}(\\d{4})", "$1****$2");
            map.put("phone", desensitizedPhoneNumber);

            memberUserScoreApi.operateScore(MemberUserScoreOperateReqDTO.builder()
                    .memberId(userId)
                    .scoreCount(scoreRuleDO.getGetScoreOnce())
                    .sourceType(ScoreSourceTypeEnum.RECOMMEND)
                    .ruleId(scoreRuleDO.getId())
                    .uniqueId(userId + "_" + message.getRecommendUserId())
                    .expireDays(scoreRuleDO.getScorePeriod())
                    .extParam(map)
                    .build());
        } catch (Exception e) {
            log.error("Recommend listener: operateScore error", e);
        }
        log.info("Recommend rule add score success,ruleID:{},userID:{}", scoreRuleDO.getId(), userId);
    }

    @Override
    public ScoreRuleTypeEnum getStrategyScoreRuleType() {
        return ScoreRuleTypeEnum.RECOMMEND;
    }
}
