package cn.iocoder.yudao.module.member.service.scoreRule;

import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.json.JSONUtil;
import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.framework.common.util.json.JsonUtils;
import cn.iocoder.yudao.framework.dict.core.util.DictFrameworkUtils;
import cn.iocoder.yudao.framework.mybatis.core.query.LambdaQuery;
import cn.iocoder.yudao.framework.mybatis.core.query.LambdaQueryWrapperX;
import cn.iocoder.yudao.framework.mybatis.core.service.AbstractService;
import cn.iocoder.yudao.framework.mybatis.core.vo.PageVO;
import cn.iocoder.yudao.module.depository.service.warehouse.WarehouseService;
import cn.iocoder.yudao.module.depository.vo.warehouse.WarehouseTreeRegionVO;
import cn.iocoder.yudao.module.member.controller.app.scoreRule.vo.AppScoreRuleBackDetailVO;
import cn.iocoder.yudao.module.member.controller.app.scoreRule.vo.AppScoreRuleListBackVO;
import cn.iocoder.yudao.module.member.controller.app.scoreRule.vo.AppScoreRuleListReqVO;
import cn.iocoder.yudao.module.member.convert.scoreRule.ScoreRuleConvert;
import cn.iocoder.yudao.module.member.dal.dataobject.scoreRule.ScoreRuleDO;
import cn.iocoder.yudao.module.member.dal.dataobject.user.MemberUserDO;
import cn.iocoder.yudao.module.member.dal.mysql.scoreRule.ScoreRuleMapper;
import cn.iocoder.yudao.module.member.dal.redis.scoreRule.ScoreRuleRedisDao;
import cn.iocoder.yudao.module.member.enums.ScoreRuleStatusEnum;
import cn.iocoder.yudao.module.member.enums.ScoreRuleTypeEnum;
import cn.iocoder.yudao.module.member.enums.TransportTypeEnum;
import cn.iocoder.yudao.module.member.enums.YesOrNoTypeEnum;
import cn.iocoder.yudao.module.member.service.user.MemberUserService;
import cn.iocoder.yudao.module.member.util.ScoreRuleGenCodeUtils;
import cn.iocoder.yudao.module.member.vo.scoreRule.*;
import cn.iocoder.yudao.module.member.vo.scoreRule.extra.ScoreRuleOrderVExtraVO;
import cn.iocoder.yudao.module.member.vo.scoreRule.extra.ScoreRuleRegisterExtraVO;
import cn.iocoder.yudao.module.member.vo.scoreRule.extra.ScoreRuleShareExtraVO;
import cn.iocoder.yudao.module.member.vo.scoreRule.extra.ScoreRulerRecommendExtraVO;
import cn.iocoder.yudao.module.system.api.user.AdminUserApi;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import org.apache.commons.collections4.CollectionUtils;
import org.apache.commons.collections4.ListUtils;
import org.apache.commons.lang3.StringUtils;
import org.jetbrains.annotations.NotNull;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.validation.annotation.Validated;

import javax.annotation.Resource;
import java.time.Instant;
import java.time.LocalDateTime;
import java.util.*;
import java.util.stream.Collectors;

import static cn.iocoder.yudao.framework.common.exception.util.ServiceExceptionUtil.exception;
import static cn.iocoder.yudao.module.member.enums.ErrorCodeConstants.*;
import static cn.iocoder.yudao.module.member.enums.TransportTypeEnum.OCEAN_LCL;
import static cn.iocoder.yudao.module.member.enums.TransportTypeEnum.SPECIAL_LINE_AIR_FREIGHT;

/**
 * 积分规则 Service 实现类
 *
 * @author 系统管理员
 */
@Service
@Validated
public class ScoreRuleServiceImpl extends AbstractService<ScoreRuleMapper, ScoreRuleDO> implements ScoreRuleService {
    private static final String PROD_REGISTER_URL = "https://app2.groupage.cn/#/pages/register/shareRegister?code=";
    private static final String UAT_REGISTER_URL = "https://apptest.groupage.cn/#/pages/register/shareRegister?code=";
    private static final String PROD_SHARE_URL = "https://app2.groupage.cn/#";
    private static final String UAT_SHARE_URL = "https://apptest.groupage.cn/#";
    @Value("${spring.profiles.active}")
    private String env;
    @Resource
    private ScoreRuleMapper scoreRuleMapper;
    @Resource
    private ScoreRuleRedisDao scoreRuleRedisDao;
    @Resource
    private WarehouseService warehouseService;
    @Resource
    private AdminUserApi adminUserApi;
    @Resource
    private ScoreRuleGenCodeUtils scoreRuleGenCodeUtils;
    @Resource
    private MemberUserService memberUserService;

    @Override
    public Long createScoreRule(ScoreRuleCreateReqVO createReqVO) {
        ScoreRuleDO scoreRule = ScoreRuleConvert.INSTANCE.convert(createReqVO);
        if (scoreRule.getStatus() != ScoreRuleStatusEnum.DISABLED.getValue() && scoreRule.getStatus() != ScoreRuleStatusEnum.ENABLED.getValue()) {
            throw exception(SCORE_RULE_STATUS_ERROR);
        }
        verifyCommon(createReqVO);
        Integer scoreRuleType = scoreRule.getType();
        verifyAndSetExtraDO(createReqVO, scoreRuleType, scoreRule);
        scoreRule.setCode(scoreRuleGenCodeUtils.generateScoreRuleCode());
        scoreRuleMapper.insert(scoreRule);
        scoreRuleRedisDao.deleteEnableScoreRule(scoreRule.getType());
        // 返回
        return scoreRule.getId();
    }

    private void verifyCommon(ScoreRuleBaseVO scoreRuleIn) {
        //校验公共入参
        if (scoreRuleIn.getStartTime().after((scoreRuleIn.getEndTime())) || scoreRuleIn.getEndTime().before(Date.from(Instant.now()))) {
            throw exception(SCORE_RULE_TIME_ERROR);
        }
        if (scoreRuleIn.getScorePeriod() <= 0) {
            throw exception(SCORE_RULE_PERIOD_ERROR);
        }
        //如果是启用，校验同一个规则下，有效期内有没有重复的规则设置，分享不校验。海运空运分开算
        if (scoreRuleIn.getStatus() == ScoreRuleStatusEnum.ENABLED.getValue() && scoreRuleIn.getType() != ScoreRuleTypeEnum.SHARE.getValue()) {
            if (scoreRuleIn.getType() == ScoreRuleTypeEnum.ORDER_V.getValue()) {
                ScoreRuleOrderVExtraVO extraOrderV = scoreRuleIn.getExtraOrderV();
                if (extraOrderV == null) {
                    throw exception(SCORE_RULE_EXTRA_ERROR);
                }
                Integer transportType = extraOrderV.getTransportType();
                if (transportType != OCEAN_LCL.getValue() && transportType != SPECIAL_LINE_AIR_FREIGHT.getValue()) {
                    throw exception(SCORE_RULE_TRANSPORT_TYPE_ERROR);
                }
                orderVUniqueCheck(extraOrderV, scoreRuleIn.getStartTime(), scoreRuleIn.getEndTime(), null);
            } else {
                scoreRuleUniqueCheck(scoreRuleIn.getType(), scoreRuleIn.getStartTime(), scoreRuleIn.getEndTime(), null);
            }
        }
    }

    /**
     * 积分规则唯一性校验（订单V值单独校验）
     *
     * @param type              类型
     * @param newScoreRuleStart 新规则的开始时间
     * @param newScoreRuleEnd   新规则的结束时间
     * @param delayId           延期规则的id
     */
    private void scoreRuleUniqueCheck(Integer type, Date newScoreRuleStart, Date newScoreRuleEnd, Long delayId) {
        LambdaQuery<ScoreRuleDO> scoreRuleDOLambdaQuery = new LambdaQuery<>();
        scoreRuleDOLambdaQuery.eq(ScoreRuleDO::getStatus, ScoreRuleStatusEnum.ENABLED.getValue())
                .eq(ScoreRuleDO::getType, type);
        if (delayId != null) {
            scoreRuleDOLambdaQuery.ne(ScoreRuleDO::getId, delayId);
        }
        List<ScoreRuleDO> scoreRuleDOS = scoreRuleMapper.selectList(scoreRuleDOLambdaQuery);
        if (scoreRuleDOS == null || scoreRuleDOS.isEmpty()) {
            return;
        }
        for (ScoreRuleDO scoreRuleDO : scoreRuleDOS) {
            if (verifyTimeOverlap(newScoreRuleStart, newScoreRuleEnd, scoreRuleDO)) {
                throw exception(SCORE_RULE_UNIQUE_CHECK_ERROR, scoreRuleDO.getCode());
            }
        }
    }

    /**
     * 订单V规则唯一性校验
     *
     * @param extraOrderV       订单v值额外信息
     * @param newScoreRuleStart 新规则的开始时间
     * @param newScoreRuleEnd   新规则的结束时间
     * @param delayId           延期时的规则id
     */
    private void orderVUniqueCheck(ScoreRuleOrderVExtraVO extraOrderV, Date newScoreRuleStart, Date newScoreRuleEnd, Long delayId) {
        LambdaQuery<ScoreRuleDO> scoreRuleDOLambdaQuery = new LambdaQuery<>();
        scoreRuleDOLambdaQuery.eq(ScoreRuleDO::getStatus, ScoreRuleStatusEnum.ENABLED.getValue())
                .eq(ScoreRuleDO::getType, ScoreRuleTypeEnum.ORDER_V.getValue());
        if (delayId != null) {
            scoreRuleDOLambdaQuery.ne(ScoreRuleDO::getId, delayId);
        }
        List<ScoreRuleDO> scoreRuleDOS = scoreRuleMapper.selectList(scoreRuleDOLambdaQuery);
        if (scoreRuleDOS == null || scoreRuleDOS.isEmpty()) {
            return;
        }
        for (ScoreRuleDO scoreRuleDO : scoreRuleDOS) {
            ScoreRuleOrderVExtraVO oldExtraVO = JSONUtil.toBean(scoreRuleDO.getExtra(), ScoreRuleOrderVExtraVO.class);
            if (verifyTimeOverlap(newScoreRuleStart, newScoreRuleEnd, scoreRuleDO) && isSameOrderVRule(oldExtraVO, extraOrderV)) {
                throw exception(SCORE_RULE_UNIQUE_CHECK_ERROR, scoreRuleDO.getCode());
            }
        }
    }

    private boolean isSameOrderVRule(ScoreRuleOrderVExtraVO oldExtraVO, ScoreRuleOrderVExtraVO extraOrderV) {
        return oldExtraVO.getTransportType().equals(extraOrderV.getTransportType()) &&
                Objects.equals(oldExtraVO.getFirstOrder(), extraOrderV.getFirstOrder()) &&
                verifyReceiveAddr(oldExtraVO, extraOrderV) &&
                verifyChannel(oldExtraVO, extraOrderV) &&
                verifyOrderEntry(oldExtraVO.getOrderEntry(), extraOrderV.getOrderEntry());
    }

    private boolean verifyChannel(ScoreRuleOrderVExtraVO oldExtraVO, ScoreRuleOrderVExtraVO extraOrderV) {
        // 运输方式空运时，判断渠道是否有交集
        if (oldExtraVO.getTransportType().equals(SPECIAL_LINE_AIR_FREIGHT.getValue())) {
            List<Long> oldChannel = oldExtraVO.getChannel();
            List<Long> newChannel = extraOrderV.getChannel();
            if (CollectionUtils.isEmpty(oldChannel) || CollectionUtils.isEmpty(newChannel)) {
                return false;
            }
            return CollectionUtils.containsAny(oldChannel, newChannel);
        }
        return true;
    }

    private boolean verifyReceiveAddr(ScoreRuleOrderVExtraVO oldExtraVO, ScoreRuleOrderVExtraVO extraOrderV) {
        List<String> oldReceiveAddrList = Arrays.asList(oldExtraVO.getReceiveAddr().split(","));
        List<String> newReceiveAddrList = new ArrayList<>();
        // 如果是前端传入的，国家，城市，仓库id从receiveAddrList中获取
        if (extraOrderV.getReceiveAddrList() != null) {
            List<List<Long>> receiveAddrList = extraOrderV.getReceiveAddrList();
            if (extraOrderV.getReceiveAddrList().isEmpty()) {
                throw exception(SCORE_RULE_RECEIVE_ADDRESS_ERROR);
            }
            for (List<Long> integers : receiveAddrList) {
                try {
                    newReceiveAddrList.add(integers.get(2).toString());
                } catch (Exception e) {
                    throw exception(SCORE_RULE_RECEIVE_ADDRESS_ERROR);
                }
            }
        } else {
            newReceiveAddrList = Arrays.asList(extraOrderV.getReceiveAddr().split(","));
        }
        // 如果有一个选的是全部，则返回true
        if (oldReceiveAddrList.contains("0") || newReceiveAddrList.contains("0")) {
            return true;
        }
        // 如果两个集合有交集，则返回true
        return CollectionUtils.containsAny(oldReceiveAddrList, newReceiveAddrList);
    }

    private boolean verifyOrderEntry(String oldOrderEntry, String newOrderEntry) {
        List<String> oldEntryList = Arrays.asList(oldOrderEntry.split(","));
        List<String> newEntryList = Arrays.asList(newOrderEntry.split(","));
        // 对订单入口进行归类后比较
        Set<String> categorizedSources1 = categorizeOrderEntry(oldEntryList);
        Set<String> categorizedSources2 = categorizeOrderEntry(newEntryList);
        // 如果一个规则归类为 "ALL"，则无论另一个是什么都算相同
        if (categorizedSources1.contains("ALL") || categorizedSources2.contains("ALL")) {
            return true;
        }
        // 仅当分类后的集合完全一致时才算相同
        return categorizedSources1.equals(categorizedSources2);
    }

    private Set<String> categorizeOrderEntry(List<String> orderEntrys) {
        Set<String> categorizedSources = new HashSet<>();
        boolean hasBackend = false;
        boolean hasClient = false;

        for (String orderEntry : orderEntrys) {
            if (orderEntry.equals("1")) {
                hasBackend = true;
            } else if (orderEntry.equals("2") || orderEntry.equals("3")) {
                hasClient = true;
            }
        }

        // 如果同时有后台和客户端，归为 "ALL"
        if (hasBackend && hasClient) {
            categorizedSources.add("ALL");
        } else {
            if (hasBackend) {
                categorizedSources.add("BACKEND");
            }
            if (hasClient) {
                categorizedSources.add("CLIENT");
            }
        }

        return categorizedSources;
    }


    /**
     * 校验规则时间区间是否有交集
     */
    private boolean verifyTimeOverlap(Date newScoreRuleStart, Date newScoreRuleEnd, ScoreRuleDO oldScoreRule) {
        return newScoreRuleStart.before(oldScoreRule.getEndTime()) && newScoreRuleEnd.after(oldScoreRule.getStartTime());
    }


    @Override
    public void updateScoreRule(ScoreRuleUpdateReqVO updateReqVO) {
        ScoreRuleDO scoreRuleDO = scoreRuleMapper.selectById(updateReqVO.getId());
        if (scoreRuleDO == null) {
            throw exception(SCORE_RULE_NOT_EXISTS);
        }
        if (scoreRuleDO.getStatus() != ScoreRuleStatusEnum.DISABLED.getValue()) {
            throw exception(SCORE_RULE_UPDATE_ERROR);
        }
        ScoreRuleDO scoreRule = ScoreRuleConvert.INSTANCE.convert(updateReqVO);
        verifyCommon(updateReqVO);
        verifyAndSetExtraDO(updateReqVO, updateReqVO.getType(), scoreRule);
        // 更新
        scoreRuleMapper.updateById(scoreRule);
        scoreRuleRedisDao.deleteEnableScoreRule(scoreRule.getType());
    }

    /**
     * 设置额外信息
     *
     * @param reqVO         请求实体
     * @param scoreRuleType 指标类型
     * @param scoreRule     需要填充额外字段的DO
     */
    private void verifyAndSetExtraDO(ScoreRuleBaseVO reqVO, Integer scoreRuleType, ScoreRuleDO scoreRule) {
        if (scoreRuleType == ScoreRuleTypeEnum.ORDER_V.getValue()) {
            ScoreRuleOrderVExtraVO extraOrderV = reqVO.getExtraOrderV();
            if (extraOrderV == null) {
                throw exception(SCORE_RULE_EXTRA_ERROR);
            }
            if (extraOrderV.getFirstOrder() != YesOrNoTypeEnum.YES.ordinal() && extraOrderV.getFirstOrder() != YesOrNoTypeEnum.NO.ordinal()) {
                throw exception(SCORE_RULE_FIRST_ORDER_ERROR);
            }
            if (extraOrderV.getTransportType() != OCEAN_LCL.getValue() && extraOrderV.getTransportType() != SPECIAL_LINE_AIR_FREIGHT.getValue()) {
                throw exception(SCORE_RULE_TRANSPORT_TYPE_ERROR);
            }
            // 国家，城市，仓库id从receiveAddrList中获取
            List<List<Long>> receiveAddrList = extraOrderV.getReceiveAddrList();
            if (receiveAddrList == null || extraOrderV.getReceiveAddrList().isEmpty()) {
                throw exception(SCORE_RULE_RECEIVE_ADDRESS_ERROR);
            }
            StringBuilder targetCountry = new StringBuilder();
            StringBuilder targetCity = new StringBuilder();
            StringBuilder receiveAddr = new StringBuilder();
            for (List<Long> integers : receiveAddrList) {
                try {
                    targetCountry.append(integers.get(0)).append(",");
                    targetCity.append(integers.get(1)).append(",");
                    receiveAddr.append(integers.get(2)).append(",");
                } catch (Exception e) {
                    throw exception(SCORE_RULE_RECEIVE_ADDRESS_ERROR);
                }
            }
            extraOrderV.setTargetCountry(targetCountry.toString());
            extraOrderV.setTargetCity(targetCity.toString());
            extraOrderV.setReceiveAddr(receiveAddr.toString());
            extraOrderV.setReceiveAddrList(null);
            if (StringUtils.isBlank(extraOrderV.getOrderEntry())) {
                throw exception(SCORE_RULE_ORDERENTRY_ERROR);
            }
            verifyOrderVRule(extraOrderV.getOrderVRule());
            scoreRule.setExtra(JSONUtil.toJsonStr(extraOrderV));
        } else if (scoreRuleType == ScoreRuleTypeEnum.REGISTER.getValue()) {
            if (reqVO.getExtraRegister() == null) {
                throw exception(SCORE_RULE_EXTRA_ERROR);
            }
            if (StringUtils.isBlank(reqVO.getExtraRegister().getRegisterPlatform())) {
                throw exception(SCORE_RULE_EXTRA_ERROR);
            }
            scoreRule.setExtra(JSONUtil.toJsonStr(reqVO.getExtraRegister()));
        } else if (scoreRuleType == ScoreRuleTypeEnum.RECOMMEND.getValue()) {
            ScoreRulerRecommendExtraVO extraRecommend = reqVO.getExtraRecommend();
            if (extraRecommend == null) {
                throw exception(SCORE_RULE_EXTRA_ERROR);
            }
            if (extraRecommend.getShareStatus() != YesOrNoTypeEnum.YES.ordinal() && extraRecommend.getShareStatus() != YesOrNoTypeEnum.NO.ordinal()) {
                throw exception(SCORE_RULE_SHARE_STATUS_ERROR);
            }
            if (StringUtils.isAnyBlank(extraRecommend.getShareContentEn(), extraRecommend.getShareContentZh())) {
                throw exception(SCORE_RULE_EXTRA_ERROR);
            }
            scoreRule.setExtra(JSONUtil.toJsonStr(extraRecommend));
        } else if (scoreRuleType == ScoreRuleTypeEnum.SHARE.getValue()) {
            ScoreRuleShareExtraVO extraShare = reqVO.getExtraShare();
            if (extraShare == null) {
                throw exception(SCORE_RULE_EXTRA_ERROR);
            }
            if (StringUtils.isAnyBlank(extraShare.getActivityDescZh(), extraShare.getActivityDescEn(), extraShare.getActivityUrl())) {
                throw exception(SCORE_RULE_EXTRA_ERROR);
            }
            if (extraShare.getShareStatus() != YesOrNoTypeEnum.YES.ordinal() && extraShare.getShareStatus() != YesOrNoTypeEnum.NO.ordinal()) {
                throw exception(SCORE_RULE_SHARE_STATUS_ERROR);
            }
            scoreRule.setExtra(JSONUtil.toJsonStr(extraShare));
        } else {
            throw exception(SCORE_RULE_NOT_EXISTS);
        }
    }

    private void verifyOrderVRule(List<ScoreRuleOrderVExtraVO.OrderVRule> extraOrderVRule) {
        if (extraOrderVRule == null || extraOrderVRule.isEmpty()) {
            throw exception(ORDERV_SCORE_RULE_ERROR);
        }
        int low = 0;
        for (ScoreRuleOrderVExtraVO.OrderVRule orderVRule : extraOrderVRule) {
            if (orderVRule.getLow() < low || orderVRule.getHigh() <= orderVRule.getLow() || orderVRule.getScore() <= 0) {
                throw exception(ORDERV_SCORE_RULE_ERROR);
            }
            low = orderVRule.getHigh();
        }
    }

    @Override
    public void deleteScoreRule(Long id) {
        ScoreRuleDO scoreRuleDO = scoreRuleMapper.selectById(id);
        if (scoreRuleDO == null) {
            throw exception(SCORE_RULE_NOT_EXISTS);
        }
        if (scoreRuleDO.getStatus() != ScoreRuleStatusEnum.DISABLED.getValue()) {
            throw exception(SCORE_RULE_DELETE_ERROR);
        }
        // 删除
        scoreRuleMapper.deleteById(id);
    }

    private void validateScoreRuleExists(Long id) {
        if (scoreRuleMapper.selectById(id) == null) {
            throw exception(SCORE_RULE_NOT_EXISTS);
        }
    }

    @Override
    public ScoreRuleBackDetailVO getScoreRule(Long id) {
        ScoreRuleDO scoreRuleDO = scoreRuleMapper.selectById(id);
        if (scoreRuleDO == null) {
            throw exception(SCORE_RULE_NOT_EXISTS);
        }
        ScoreRuleBackDetailVO scoreRuleBackDetailVO = new ScoreRuleBackDetailVO();
        BeanUtils.copyProperties(scoreRuleDO, scoreRuleBackDetailVO);
        setExtraVO(scoreRuleBackDetailVO);
        String creator = adminUserApi.getUserName(scoreRuleDO.getCreator());
        scoreRuleBackDetailVO.setCreator(creator);
        String updater = adminUserApi.getUserName(scoreRuleDO.getUpdater());
        scoreRuleBackDetailVO.setUpdater(updater);
        return scoreRuleBackDetailVO;
    }


    @Override
    public List<ScoreRuleBackVO> getScoreRuleList(Collection<Long> ids) {
        List<ScoreRuleDO> scoreRuleDOS = scoreRuleMapper.selectBatchIds(ids);
        return ScoreRuleConvert.INSTANCE.convertList(scoreRuleDOS);
    }

    @Override
    public PageResult<ScoreRuleBackVO> getScoreRulePage(ScoreRuleQueryVO query, PageVO page) {
        PageResult<ScoreRuleDO> pageResult = scoreRuleMapper.selectPage(page, query);
        pageResult.getList().forEach(scoreRuleDO -> {
            String creator = adminUserApi.getUserName(scoreRuleDO.getCreator());
            scoreRuleDO.setCreator(creator);
            String updater = adminUserApi.getUserName(scoreRuleDO.getUpdater());
            scoreRuleDO.setUpdater(updater);
        });
        return ScoreRuleConvert.INSTANCE.convertPage(pageResult);
    }

    /**
     * 设置返回VO额外信息
     *
     * @param scoreRuleBackDetailVO VO
     */
    private void setExtraVO(ScoreRuleBackDetailVO scoreRuleBackDetailVO) {
        if (scoreRuleBackDetailVO.getType() == ScoreRuleTypeEnum.ORDER_V.getValue()) {
            ScoreRuleOrderVExtraVO orderVExtraVO = JSONUtil.toBean(scoreRuleBackDetailVO.getExtra(), ScoreRuleOrderVExtraVO.class);
            List<List<Long>> receiveAddrList = getReceiveAddrList(orderVExtraVO);
            orderVExtraVO.setReceiveAddrList(receiveAddrList);
            scoreRuleBackDetailVO.setExtraOrderV(orderVExtraVO);
        } else if (scoreRuleBackDetailVO.getType() == ScoreRuleTypeEnum.REGISTER.getValue()) {
            scoreRuleBackDetailVO.setExtraRegister(JSONUtil.toBean(scoreRuleBackDetailVO.getExtra(), ScoreRuleRegisterExtraVO.class));
        } else if (scoreRuleBackDetailVO.getType() == ScoreRuleTypeEnum.RECOMMEND.getValue()) {
            scoreRuleBackDetailVO.setExtraRecommend(JSONUtil.toBean(scoreRuleBackDetailVO.getExtra(), ScoreRulerRecommendExtraVO.class));
        } else if (scoreRuleBackDetailVO.getType() == ScoreRuleTypeEnum.SHARE.getValue()) {
            scoreRuleBackDetailVO.setExtraShare(JSONUtil.toBean(scoreRuleBackDetailVO.getExtra(), ScoreRuleShareExtraVO.class));
        } else {
            throw exception(SCORE_RULE_NOT_EXISTS);
        }
    }

    private @NotNull List<List<Long>> getReceiveAddrList(ScoreRuleOrderVExtraVO orderVExtraVO) {
        List<List<Long>> receiveAddrList = new ArrayList<>();
        String[] receiveAddrStrs = orderVExtraVO.getReceiveAddr().split(",");
        String[] targetCityStrs = orderVExtraVO.getTargetCity().split(",");
        String[] targetCountryStrs = orderVExtraVO.getTargetCountry().split(",");
        for (int i = 0; i < receiveAddrStrs.length; i++) {
            List<Long> list = new ArrayList<>();
            list.add(Long.parseLong(targetCountryStrs[i]));
            list.add(Long.parseLong(targetCityStrs[i]));
            list.add(Long.parseLong(receiveAddrStrs[i]));
            receiveAddrList.add(list);
        }
        return receiveAddrList;
    }

    @Override
    public List<ScoreRuleDO> getScoreRuleList(ScoreRuleQueryVO query) {
        return scoreRuleMapper.selectList(query);
    }

    @Override
    public void updateStatus(ScoreRuleStatusReqVO scoreRuleStatusReqVO) {
        ScoreRuleDO scoreRuleDO = scoreRuleMapper.selectById(scoreRuleStatusReqVO.getId());
        if (scoreRuleDO == null) {
            throw exception(SCORE_RULE_NOT_EXISTS);
        }
        Integer oldStatus = scoreRuleDO.getStatus();
        ScoreRuleDO upScoreRuleDO = new ScoreRuleDO();
        upScoreRuleDO.setId(scoreRuleStatusReqVO.getId());
        if (oldStatus == ScoreRuleStatusEnum.ENABLED.getValue() && scoreRuleStatusReqVO.getStatus() == ScoreRuleStatusEnum.CLOSED.getValue()) {
            upScoreRuleDO.setStatus(scoreRuleStatusReqVO.getStatus());
        } else if (oldStatus == ScoreRuleStatusEnum.DISABLED.getValue() && scoreRuleStatusReqVO.getStatus() == ScoreRuleStatusEnum.ENABLED.getValue()) {
            // 如果结束时间比当前时间早，不允许启用
            if (scoreRuleDO.getEndTime() != null && scoreRuleDO.getEndTime().before(new Date())) {
                throw exception(SCORE_RULE_TIME_ERROR);
            }
            //如果是启用，校验同一个规则下，有效期内有没有重复的规则设置，分享不校验
            if (scoreRuleDO.getType() != ScoreRuleTypeEnum.SHARE.getValue()) {
                if (scoreRuleDO.getType() == ScoreRuleTypeEnum.ORDER_V.getValue()) {
                    ScoreRuleOrderVExtraVO extraOrderV = JSONUtil.toBean(scoreRuleDO.getExtra(), ScoreRuleOrderVExtraVO.class);
                    orderVUniqueCheck(extraOrderV, scoreRuleDO.getStartTime(), scoreRuleDO.getEndTime(), null);
                } else {
                    scoreRuleUniqueCheck(scoreRuleDO.getType(), scoreRuleDO.getStartTime(), scoreRuleDO.getEndTime(), null);
                }
            }
            upScoreRuleDO.setStatus(scoreRuleStatusReqVO.getStatus());
        } else {
            throw exception(SCORE_RULE_STATUS_ERROR);
        }
        scoreRuleMapper.updateById(upScoreRuleDO);
        scoreRuleRedisDao.deleteEnableScoreRule(scoreRuleDO.getType());
    }

    @Override
    public Long copyScoreRule(Long id) {
        ScoreRuleDO scoreRuleDO = scoreRuleMapper.selectById(id);
        if (scoreRuleDO == null) {
            throw exception(SCORE_RULE_NOT_EXISTS);
        }
        scoreRuleDO.setStatus(ScoreRuleStatusEnum.DISABLED.getValue());
        scoreRuleDO.setId(null);
        scoreRuleDO.setCreateTime(null);
        scoreRuleDO.setUpdateTime(null);
        scoreRuleDO.setUpdater(null);
        scoreRuleDO.setCode(scoreRuleGenCodeUtils.generateScoreRuleCode());
        scoreRuleMapper.insert(scoreRuleDO);
        return scoreRuleDO.getId();
    }

    @Override
    public void delayScoreRule(ScoreDelayReqVO delayReqVO) {
        ScoreRuleDO scoreRuleDO = scoreRuleMapper.selectById(delayReqVO.getId());
        if (scoreRuleDO == null) {
            throw exception(SCORE_RULE_NOT_EXISTS);
        }
        if (scoreRuleDO.getStatus() != ScoreRuleStatusEnum.ENABLED.getValue()) {
            throw exception(SCORE_RULE_STATUS_NOT_ENABLE);
        }
        Instant now = Instant.now();
        // 结束时间不能小于当前时间
        if (!delayReqVO.getEndTime().toInstant().isAfter(now) || !delayReqVO.getEndTime().toInstant().isAfter(scoreRuleDO.getStartTime().toInstant())) {
            throw exception(SCORE_RULE_TIME_ERROR);
        }
        // 新时间区间规则唯一校验
        if (scoreRuleDO.getType() == ScoreRuleTypeEnum.ORDER_V.getValue()) {
            ScoreRuleOrderVExtraVO extraOrderV = JSONUtil.toBean(scoreRuleDO.getExtra(), ScoreRuleOrderVExtraVO.class);
            orderVUniqueCheck(extraOrderV, scoreRuleDO.getStartTime(), delayReqVO.getEndTime(), scoreRuleDO.getId());
        } else {
            scoreRuleUniqueCheck(scoreRuleDO.getType(), scoreRuleDO.getStartTime(), delayReqVO.getEndTime(), scoreRuleDO.getId());
        }
        ScoreRuleDO upScoreRuleDO = new ScoreRuleDO();
        upScoreRuleDO.setId(delayReqVO.getId());
        upScoreRuleDO.setEndTime(delayReqVO.getEndTime());
        scoreRuleMapper.updateById(upScoreRuleDO);
    }

    @Override
    public ScoreRuleDO getEnabledOrderVScoreRuleByTransportType(Integer transportType) {
        if (transportType != OCEAN_LCL.getValue() && transportType != SPECIAL_LINE_AIR_FREIGHT.getValue()) {
            throw exception(SCORE_RULE_TRANSPORT_TYPE_ERROR);
        }
        LambdaQuery<ScoreRuleDO> lambdaQuery = new LambdaQuery<>();
        // 查询状态为启用，类型为订单V值，当前时间在规则时间区间内的规则
        lambdaQuery.eq(ScoreRuleDO::getStatus, ScoreRuleStatusEnum.ENABLED.getValue())
                .eq(ScoreRuleDO::getType, ScoreRuleTypeEnum.ORDER_V.getValue())
                .lt(ScoreRuleDO::getStartTime, LocalDateTime.now())
                .gt(ScoreRuleDO::getEndTime, LocalDateTime.now());
        List<ScoreRuleDO> scoreRuleDOS = scoreRuleMapper.selectList(lambdaQuery);
        if (!CollectionUtil.isEmpty(scoreRuleDOS)) {
            for (ScoreRuleDO scoreRuleDO : scoreRuleDOS) {
                if (Objects.equals(JSONUtil.toBean(scoreRuleDO.getExtra(), ScoreRuleOrderVExtraVO.class).getTransportType(), transportType)) {
                    return scoreRuleDO;
                }
            }
        }
        return null;
    }

    @Override
    public ScoreRuleDO getEnableScoreRuleByType(ScoreRuleTypeEnum scoreRuleType) {
        ScoreRuleDO enableScoreRule;
        enableScoreRule = scoreRuleRedisDao.getEnableScoreRule(scoreRuleType);
        if (enableScoreRule == null) {
            enableScoreRule = selectOne(new LambdaQuery<ScoreRuleDO>()
                    .eq(ScoreRuleDO::getType, scoreRuleType.getValue())
                    .eq(ScoreRuleDO::getStatus, ScoreRuleStatusEnum.ENABLED.getValue())
                    .lt(ScoreRuleDO::getStartTime, LocalDateTime.now())
                    .gt(ScoreRuleDO::getEndTime, LocalDateTime.now()));
            if (enableScoreRule == null) {
                return null;
            }
            scoreRuleRedisDao.setEnableScoreRule(enableScoreRule);
        }
        return enableScoreRule;
    }

    @Override
    public List<WarehouseTreeRegionSimpleVO> getWarehouseTreeRegionListSimple() {
        List<WarehouseTreeRegionVO> list = ListUtils.sum(
                warehouseService.getWarehouseTreeRegionList(1),
                warehouseService.getWarehouseTreeRegionList(2)
        );
        return list.stream().map(this::convertToSimpleVO).collect(Collectors.toList());
    }

    private WarehouseTreeRegionSimpleVO convertToSimpleVO(WarehouseTreeRegionVO regionVO) {
        WarehouseTreeRegionSimpleVO simpleVO = new WarehouseTreeRegionSimpleVO();
        simpleVO.setId(regionVO.getGuojia());
        simpleVO.setLabelZh(regionVO.getGuojiaName().trim());
        simpleVO.setLabelEn(regionVO.getChildren().isEmpty() ? "" : regionVO.getChildren().get(0).getGuojiaNameEn());

        // 构建城市和仓库的树状结构
        Map<Long, WarehouseTreeRegionSimpleVO.WarehouseCity> cityMap = new LinkedHashMap<>();

        for (WarehouseTreeRegionVO child : regionVO.getChildren()) {
            WarehouseTreeRegionSimpleVO.WarehouseCity city = cityMap
                    .computeIfAbsent(child.getShi(), key -> {
                        WarehouseTreeRegionSimpleVO.WarehouseCity newCity = new WarehouseTreeRegionSimpleVO.WarehouseCity();
                        newCity.setId(child.getShi());
                        newCity.setLabelZh(child.getShiName().trim());
                        newCity.setLabelEn(child.getShiNameEn());
                        newCity.setChildren(new ArrayList<>());
                        return newCity;
                    });

            WarehouseTreeRegionSimpleVO.Warehouse warehouse = new WarehouseTreeRegionSimpleVO.Warehouse();
            warehouse.setId(child.getId());
            warehouse.setLabelZh(child.getTitleZh().trim());
            warehouse.setLabelEn(child.getTitleEn());
            city.getChildren().add(warehouse);
        }

        simpleVO.setChildren(new ArrayList<>(cityMap.values()));
        return simpleVO;
    }

    @Override
    public List<AppScoreRuleListBackVO> appGetScoreRuleList(AppScoreRuleListReqVO reqVO) {
        // 开启，且活动开始时间在当前时间前，结束时间在当前时间后
        LambdaQueryWrapperX<ScoreRuleDO> queryWrapperX = new LambdaQueryWrapperX<>();
        queryWrapperX.eq(ScoreRuleDO::getStatus, ScoreRuleStatusEnum.ENABLED.getValue())
                .lt(ScoreRuleDO::getStartTime, new Date())
                .gt(ScoreRuleDO::getEndTime, new Date());
        List<ScoreRuleDO> scoreRuleDOS = scoreRuleMapper.selectList(queryWrapperX);
        if (CollectionUtils.isNotEmpty(scoreRuleDOS)) {
            List<AppScoreRuleListBackVO> appScoreRuleListBackVOS = new ArrayList<>();
            for (ScoreRuleDO scoreRuleDO : scoreRuleDOS) {
                // 校验平台入口
                if (!Arrays.asList(scoreRuleDO.getShowPlatform().split(",")).contains(String.valueOf(reqVO.getPlatform()))) {
                    continue;
                }
                AppScoreRuleListBackVO appScoreRuleListBackVO = new AppScoreRuleListBackVO();
                BeanUtils.copyProperties(scoreRuleDO, appScoreRuleListBackVO);
                // 推荐和分享设置额外字段
                if (scoreRuleDO.getType() == ScoreRuleTypeEnum.RECOMMEND.getValue()) {
                    ScoreRulerRecommendExtraVO scoreRulerRecommendExtraVO = JsonUtils.parseObject(scoreRuleDO.getExtra(), ScoreRulerRecommendExtraVO.class);
                    appScoreRuleListBackVO.setExtraRecommend(scoreRulerRecommendExtraVO);
                    MemberUserDO memberUserDO = memberUserService.getUser(reqVO.getMemberId());
                    if (memberUserDO != null && !"pro".equals(env)) {
                        appScoreRuleListBackVO.setRecommendUrl(UAT_REGISTER_URL + memberUserDO.getCode());
                    } else if (memberUserDO != null) {
                        appScoreRuleListBackVO.setRecommendUrl(PROD_REGISTER_URL + memberUserDO.getCode());
                    }
                } else if (scoreRuleDO.getType() == ScoreRuleTypeEnum.SHARE.getValue()) {
                    ScoreRuleShareExtraVO scoreRuleShareExtraVO = JsonUtils.parseObject(scoreRuleDO.getExtra(), ScoreRuleShareExtraVO.class);
                    MemberUserDO memberUserDO = memberUserService.getUser(reqVO.getMemberId());
                    String activityUrl = scoreRuleShareExtraVO.getActivityUrl();
                    if (memberUserDO != null && !"pro".equals(env)) {
                        scoreRuleShareExtraVO.setActivityUrl(UAT_SHARE_URL + activityUrl);
                    } else if (memberUserDO != null) {
                        scoreRuleShareExtraVO.setActivityUrl(PROD_SHARE_URL + activityUrl);
                    }
                    appScoreRuleListBackVO.setExtraShare(scoreRuleShareExtraVO);
                }
                String type = appScoreRuleListBackVO.getType().toString();
                appScoreRuleListBackVO.setTypeZh(DictFrameworkUtils.getDictDataFromCache("score_rule_type", type).getLabel());
                appScoreRuleListBackVO.setTypeEn(DictFrameworkUtils.getDictDataFromCache("score_rule_type", type).getLabelEn());
                appScoreRuleListBackVOS.add(appScoreRuleListBackVO);
            }
            return appScoreRuleListBackVOS;
        }
        return null;
    }

    @Override
    public AppScoreRuleBackDetailVO appGetScoreRule(Long id) {
        ScoreRuleBackDetailVO scoreRule = getScoreRule(id);
        AppScoreRuleBackDetailVO appScoreRuleBackDetailVO = new AppScoreRuleBackDetailVO();
        BeanUtils.copyProperties(scoreRule, appScoreRuleBackDetailVO);
        String type = appScoreRuleBackDetailVO.getType().toString();
        appScoreRuleBackDetailVO.setTypeZh(DictFrameworkUtils.getDictDataFromCache("score_rule_type", type).getLabel());
        appScoreRuleBackDetailVO.setTypeEn(DictFrameworkUtils.getDictDataFromCache("score_rule_type", type).getLabelEn());
        return appScoreRuleBackDetailVO;
    }

    @Override
    public Boolean getScoreRuleSwitch() {
        return scoreRuleRedisDao.getScoreRuleSwitch();
    }

    @Override
    public Boolean setScoreRuleSwitch(SwitchReqVo switchReqVo) {
        return scoreRuleRedisDao.setScoreRuleSwitch(switchReqVo.getSwitchState());
    }

    @Override
    public List<ScoreRuleDO> getEnableOrderVRule() {
        LambdaQueryWrapper<ScoreRuleDO> queryWrapper = new LambdaQuery<ScoreRuleDO>()
                .eq(ScoreRuleDO::getType, ScoreRuleTypeEnum.ORDER_V.getValue())
                .eq(ScoreRuleDO::getStatus, ScoreRuleStatusEnum.ENABLED.getValue())
                .lt(ScoreRuleDO::getStartTime, LocalDateTime.now())
                .gt(ScoreRuleDO::getEndTime, LocalDateTime.now());
        List<ScoreRuleDO> scoreRuleDOS = scoreRuleMapper.selectList(queryWrapper);
        return scoreRuleDOS;
    }
}
